/*
 * ArrayList.cpp implements the ArrayList class behavior declared in ArrayList.h.
 *
 * Today's version is able to resize the array to grow it as needed if it
 * becomes full. The various member functions don't check indexes to see if they are out
 * of bounds.  Also the list can store only ints.  We will address these issues later.
 */

#include <string>
#include "ArrayList.h"

using namespace std;

// helper that sets all elements of an array to 0
static void zeroOut(int* a, int length);

ArrayList::ArrayList() {}

ArrayList::~ArrayList() {
    delete[] m_elements;
}

void ArrayList::add(int value) {
    checkResize();
    m_elements[m_size] = value;
    m_size++;
}

void ArrayList::clear() {
    zeroOut(m_elements, m_capacity);
    m_size = 0;
}

int ArrayList::get(int index) const {
    return m_elements[index];
}

void ArrayList::insert(int index, int value) {
    checkResize();

    // shift right to make room
    for (int i = m_size; i > index; i--) {
        m_elements[i] = m_elements[i - 1];
    }
    m_elements[index] = value;
    m_size++;
}

bool ArrayList::isEmpty() const {
    return m_size == 0;
}

void ArrayList::remove(int index) {
    for (int i = index; i < m_size; i++) {
        m_elements[i] = m_elements[i + 1];
    }
    m_size--;
}

void ArrayList::set(int index, int value) {
    m_elements[index] = value;
}

int ArrayList::size() const {
    return m_size;
}

string ArrayList::toString() const {
    string s = "[";
    if (!isEmpty()) {
        s += to_string(m_elements[0]);
        for (int i = 1; i < m_size; i++) {
            s += ", ";
            s += to_string(m_elements[i]);
        }
    }
    s += "]";
    return s;
}

void ArrayList::checkResize() {
    if (m_size == m_capacity) {
        // out of space; resize
        int* bigDaddy = new int[m_capacity * 2];
        for (int i = 0; i < m_size; i++) {
            bigDaddy[i] = m_elements[i];
        }
        delete[] m_elements;   // free old array's memory
        m_elements = bigDaddy;
        m_capacity *= 2;
    }
}

// helper to set all m_elements of an int array to 0.
static void zeroOut(int* a, int length) {
    for (int i = 0; i < length; i++) {
        a[i] = 0;
    }
}
